/* Copyright 2020, Michele Santullo
 * This file is part of user-gcc.
 *
 * User-gcc is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * User-gcc is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with user-gcc.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include "env_base.hpp"
#include <unordered_map>
#include <memory>
#include <mutex>

namespace duck {
	class EnvFake : public EnvBase {
		typedef std::unordered_map<std::string_view, std::string_view> map_type;
	public:
		typedef map_type::const_iterator const_iterator;

		explicit EnvFake (bool starts_empty=false);
		~EnvFake() noexcept = default;

		EnvFake& operator= (const EnvBase& other);

		const_iterator begin() const;
		const_iterator cbegin() const;
		const_iterator end() const;
		const_iterator cend() const;

		virtual std::mutex& mutex() const noexcept override;

	protected:
		virtual void set_all_into_implem (EnvBase& other) const override;
		virtual void clear_implem(Mutex& m) override;
		virtual std::optional<std::string_view> get_implem (std::string_view name, Mutex& m) const override;
		virtual std::string_view get_implem (std::string_view name, std::string_view def, Mutex& m) const override;
		virtual void set_implem (std::string_view name, std::string_view value, Mutex& m) override;
		virtual void unset_implem (std::string_view name, Mutex& m) override;
		virtual bool is_set_implem (std::string_view name, Mutex& m) const override;
		virtual size_type size_implem (Mutex& m) const noexcept override;
		virtual std::unique_ptr<char*[]> to_c_array_implem (Mutex& m) const override;

		virtual bool is_global() const noexcept override { return false; }
		virtual uint32_t class_identifier() const noexcept override;

	private:
		std::unordered_map<std::string_view, std::unique_ptr<char[]>> m_map_strong_cpy;
		map_type m_map;
		mutable std::mutex m_mutex;
	};
} //namespace duck
