/* Copyright 2020, Michele Santullo
 * This file is part of user-gcc.
 *
 * User-gcc is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * User-gcc is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with user-gcc.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include "env_base.hpp"
#include <numeric>
#include <mutex>
#include <memory>
#include <cassert>

namespace duck {
	template <typename E>
	inline std::unique_ptr<char*[]> env_to_c_array (const E& env, typename E::size_type count) {
		using std::pair;
		using std::string_view;
		typedef typename E::size_type size_type;

		const size_type all_str_size = std::accumulate(
			env.begin(),
			env.end(),
			size_type{0},
			[](size_type s, const pair<string_view, string_view>& itm) {
				return s + itm.first.size() + itm.second.size() + 1 + 1;
			}
		);

		auto buff = std::make_unique<char*[]>(count + 1 + (all_str_size + sizeof(char*) - 1) / sizeof(char*));
		size_type z = 0;
		char* curr_str = reinterpret_cast<char*>(buff.get() + count + 1);
		for (const auto& itm : env) {
			std::copy(itm.first.begin(), itm.first.end(), curr_str);
			curr_str[itm.first.size()] = '=';
			std::copy(itm.second.begin(), itm.second.end(), curr_str + itm.first.size() + 1);
			const std::size_t curr_size = itm.first.size() + itm.second.size() + 1 + 1;
			curr_str[curr_size - 1] = '\0';
			assert(z < count);
			buff[z] = curr_str;

			curr_str += curr_size;
			++z;
		}

		buff[count] = nullptr;
		return buff;
	}
} //namespace duck
