/* Copyright 2020, Michele Santullo
 * This file is part of user-gcc.
 *
 * User-gcc is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * User-gcc is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with user-gcc.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <iostream>
#include <cxxopts.hpp>
#include <string>
#include "config.h"
#include "list_profiles.hpp"
#include "run_context.hpp"

namespace {
	const char g_simple_colours[] =
		"GOOD=\033[32;01m\n"
		"WARN=\033[33;01m\n"
		"BAD=\033[31;01m\n"
		"HILITE=\033[36;01m\n"
		"BRACKET=\033[34;01m\n"
		"NORMAL=\033[0m\n"
	;
} //unnamed namespace

int main(int argc, char* argv[]) {
	using std::string;
	cxxopts::Options options(PROJECT_NAME, "Change the current compiler profile, or give info about profiles");
	options.add_options()
		("h,help", "Show this help and quit")
		("version", "Print version info")
		("l,list-profiles", "Print a list of available profiles") //, cxxopts::value<string>()->default_value(""))
	;
	const int orig_argc = argc;
	auto command = options.parse(argc, argv);

	if (command.count("help") or 1 == orig_argc) {
		std::cout << options.help() << std::endl;
		return 0;
	}
	if (command.count("version")) {
		std::cout << PROJECT_NAME << " v" << PROJECT_VERSION << '\n';
		std::cout << "Copyright (C) " COPYRIGHT_YEAR "  Michele Santullo\n";
		std::cout << "This is free software; see the source for copying conditions.  There is NO\n";
		std::cout << "warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n";
		std::cout << "To obtain source code please visit:\n -> " << PROJECT_REPO_URL << '\n';
		return 0;
	}

	duck::RunContext syspaths;
	syspaths.add_to_env(g_simple_colours);
	syspaths.env().set("argv0", PROJECT_NAME);
#if !defined(NDEBUG)
	std::cout << syspaths << '\n';
#endif

	if (command.count("list-profiles")) {
		duck::list_profiles(syspaths);
	}

	return 0;
}
