/* Copyright 2020, Michele Santullo
 * This file is part of user-gcc.
 *
 * User-gcc is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * User-gcc is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with user-gcc.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#define USE_FAKE_ENV

#if defined(USE_FAKE_ENV)
#	include "env_fake.hpp"
#else
#	include "env_real.hpp"
#endif
#include <filesystem>
#include <string>
#if !defined(NDEBUG)
#	include <iostream>
#endif
#include <unordered_map>
#include <string_view>
#include <optional>

namespace duck {
	class RunContext {
	public:
		typedef std::filesystem::path path_type;
#if defined(USE_FAKE_ENV)
		typedef EnvFake env_type;
#else
		typedef EnvReal env_type;
#endif

		RunContext();
		~RunContext() noexcept = default;

		env_type& env() noexcept { return m_env; }
		const env_type& env() const noexcept { return m_env; }

		void add_to_env (std::string_view colours);

		const path_type& root() const;
		const path_type& eroot() const;
		const path_type& eprefix() const;
		const path_type& env_d() const;
		const path_type& gcc_env_d() const;

	private:
		env_type m_env;
		path_type m_root;
		path_type m_eprefix;
		path_type m_eroot;
		path_type m_env_d;
		path_type m_gcc_env_d;
	};

#if !defined(NDEBUG)
	std::ostream& operator<< (std::ostream&, const RunContext& sp);
#endif

#if defined(USE_FAKE_ENV)
#	undef USE_FAKE_ENV
#endif
} //namespace duck
